import * as utils from './utils'
import { invoke, invokeMethod, subscribe, subscribeHandler, webSubscribe, publish } from './mop_bridge'
/**
 * 这是MOPClient模板 目前仅用于
 * @module mop
 */

let currentRoute = ''
let tabBar = null

/**
 * MOPClient类,提供MOP小程序WebView的JSSDK交互功能
 * 代码示例:
 * <code>
 window.fc.miniProgram.navigateBack({
   success: function(res) {
     console.log('navigateBack callback:');
     console.log(res);
     alert(JSON.stringify(res))
   }
 });
 * </code>
 */

let eventHandlers = []

class MOPClient {
  constructor() {
    console.log('MOPClient')
    try {
      subscribe('webviewEvent', (data) => {
        eventHandlers.forEach((handler) => {
          handler(data.data)
        })
      })
      subscribe('webviewCurrentRoute', (data) => {
        console.log(data)
        currentRoute = data.route
        tabBar = data.tabBar
      })
      publish('onWebviewEvent', {
        key: 'getCurrentRoute',
      })
    } catch (e) { }
  }

  checkUrl (apiName, params) {
    // Mark: 未实现传递 tabbar 的情况不校验
    if (tabBar === null) {
      return true
    }
    if (params.url.startsWith('/')) {
      params.url = params.url.slice(1)
    }
    if ((apiName === 'redirectTo' || apiName === 'navigateTo') &&
      (tabBar &&
        tabBar.list.find(e => {
          return params.url.indexOf(e.pagePath) === 0
        }))) {
      console.error(apiName, params, apiName + `:can not ${apiName} to a tabbar page`)
      return false
    } else if (apiName === 'switchTab') {
      if (!tabBar || (!tabBar.list.find(e => {
        return params.url.indexOf(e.pagePath) === 0
      }))) {
        console.error(apiName, params, apiName + `:fail can not switch to no-tabBar page`)
        return false
      }
    }
    return true
  }

  /**
   * 关闭当前页面，返回上一页面或多级页面。
   * @param {Object} params {
   *  delta: 1,
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */
  navigateBack(params = {}) {
    const data = params || {}
    data.name = 'navigateBack'
    data.arg = {
      delta: params.delta || 1,
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 保留当前页面，跳转到应用内的某个页面。但是不能跳到 tabbar 页面。
   * @param {Object} params {
   *  url: '',
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */
  navigateTo(params) {
    const data = params || {}
    if (!params.url) {
      throw new Error('Invalid parameter url')
    }
    params.url = utils.getRealRoute(currentRoute, params.url)
    params.url = utils.encodeUrlQuery(params.url)
    data.name = 'navigateTo'
    data.arg = {
      url: params.url || '',
    }

    if (this.checkUrl(data.name, data.arg)) {
      invokeMethod('invokeMiniProgramAPI', data)
    }
  }

  /**
   * 关闭当前页面，跳转到应用内的某个页面。但是不允许跳转到 tabbar 页面。
   * @param {Object} params {
   *  url: '',
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */
  redirectTo(params) {
    const data = params || {}
    if (!params.url) {
      throw new Error('Invalid parameter url')
    }
    params.url = utils.getRealRoute(currentRoute, params.url)
    params.url = utils.encodeUrlQuery(params.url)
    data.name = 'redirectTo'
    data.arg = {
      url: params.url || '',
    }

    if (this.checkUrl(data.name, data.arg)) {
      invokeMethod('invokeMiniProgramAPI', data)
    }
  }

  /**
   * 跳转到 tabBar 页面，并关闭其他所有非 tabBar 页面。
   * @param {Object} params {
   *  url: '',
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */
  switchTab(params) {
    const data = params || {}
    if (!params.url) {
      throw new Error('Invalid parameter url')
    }
    params.url = utils.getRealRoute(currentRoute, params.url)
    params.url = utils.encodeUrlQuery(params.url)
    data.name = 'switchTab'
    data.arg = {
      url: params.url || '',
    }
    if (this.checkUrl(data.name, data.arg)) {
      invokeMethod('invokeMiniProgramAPI', data)
    }
  }

  /**
   * 关闭所有页面，打开到应用内的某个页面
   * @param {Object} params {
   *  url: '',
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */
  reLaunch(params) {
    const data = params || {}
    if (!params.url) {
      throw new Error('Invalid parameter url')
    }
    params.url = utils.getRealRoute(currentRoute, params.url)
    params.url = utils.encodeUrlQuery(params.url)
    data.name = 'reLaunch'
    data.arg = {
      url: params.url || '',
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 发送消息
   * @param {Object} params {
   *  data: '',
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */
  postMessage(params) {
    const data = params || {}
    data.name = 'postMessage'
    data.arg = params.data || {}
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 取当前运行环境
   * @param {Object} params {
   *  success: function(res) {}
   * }
   */
  getEnv(params) {
    if (params && typeof params === 'object' && typeof params.success === 'function') {
      params.success({
        env: window.__fcjs_environment || {},
      })
    } else if (typeof params === 'function') {
      params({
        env: window.__fcjs_environment || {},
      })
    } else {
      console.log('params invalid')
    }
  }

  /**
   * 下载文件
   * @param {Object} params {
   *  url: '',
   *  header: '',
   *  timeout: 3000,
   *  filePath: ''
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */
  downloadFile(params) {
    const data = params || {}
    data.name = 'downloadFile'
    data.arg = {
      url: params.url,
      header: params.header,
      timeout: params.timeout,
      filePath: params.filePath,
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 打开文件，预览文件
   * @param {*} params {
   *  fileType: 'pdf',
   *  filePath: ''
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */
  openDocument(params) {
    const data = params || {}
    data.name = 'openDocument'
    data.arg = {
      filePath: params.filePath,
      fileType: params.fileType,
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 选择图片
   * @param {*} params {
   * count: 1, // 默认9
   * sizeType: ['original', 'compressed'], // 可以指定是原图还是压缩图，默认二者都有
   * sourceType: ['album', 'camera'], // 可以指定来源是相册还是相机，默认二者都有
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */
  chooseImage(params) {
    const data = params || {}
    data.name = 'chooseImage'
    data.arg = {
      count: params.count || 9,
      sizeType: params.sizeType || ['original', 'compressed'],
      sourceType: params.sourceType || ['album', 'camera'],
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 获取本地图片 base64
   * @param {*} params
   * path: 本地图片路径
   */
  getLocalImgData(params) {
    const data = params || {}
    data.name = 'getLocalImgData'
    data.arg = {
      path: params.path,
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 获取小程序信息
   * @param {callback} cb function(res) {}
   */
  getAppletInfo(params) {
    const data = params || {}
    data.name = 'getAppletInfo'
    data.arg = {}
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 关闭小程序
   */
  close(params) {
    const data = params || {}
    data.name = 'close'
    data.arg = {}
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 小程序跳转
   * @param {*} params {
   * appId: string, // 要打开的小程序 appId
   * path: string, //打开的页面路径
   * extraData: Object, // 需要传递给目标小程序的数据,App.onLaunch，App.onShow 中获取到这份数据
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */
  navigateToMiniProgram(params) {
    const data = params || {}
    data.name = 'navigateToMiniProgram'
    data.arg = {
      appId: params.appId,
      path: params.path,
      extraData: params.extraData,
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 返回上一个小程序
   * @param {*} params {
   * extraData: Object, // 需要传递给目标小程序的数据,App.onLaunch，App.onShow 中获取到这份数据
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */
  navigateBackMiniProgram(params) {
    const data = params || {}
    data.name = 'navigateBackMiniProgram'
    data.arg = {
      extraData: params.extraData,
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 判断 H5 是否能返回上一页
   * @param {Object} params
   */
  canGoBack(params) {
    const data = params || {}
    data.name = 'canGoBack'
    data.arg = {}
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
  * 判断小程序页面是否能返回上一页
  * @param {Object} params
  */
  canNavigateBack(params) {
    const data = params || {}
    data.name = 'canNavigateBack'
    data.arg = {}
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
  * webview 页面设置 navigation title
  * @param {Object} params
  *   title: String
  * }
  */
  setNavigationBarTitle(params) {
    const data = params || {}
    data.name = 'setNavigationBarTitle'
    data.arg = {
      title: String(params.title || ''),
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * @description 获取本地缓存
   * @param {Object} params
   * @param {String} params.key
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   */
  getStorage(params) {
    if (typeof params.key !== 'string') {
      throw new Error('Invalid parameter key')
    }
    const data = params || {}
    data.name = 'getStorage'
    data.arg = {
      key: params.key,
    }
    invokeMethod('invokeMiniProgramAPI', data, {
      beforeSuccess: function(res) {
        if (typeof res.data === 'string') {
          try {
            const result = JSON.parse(res.data)
            if (result._type) {
              res.data = result.data
            } else {
              res.data = result
            }
            if (result._type === 'Date') {
              res.data = new Date(res.data)
            }
          } catch (error) { }
        }
        return res
      },
    })
  }

  /**
   * @description 获取本地缓存的数据
   * @param {Object} params
   * @param {String} params.key
   * @param {String} params.data
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   */
  setStorage(params) {
    if ((typeof params.key !== 'string')) {
      console.error('parameters error')
      return
    }
    let _type = typeof params.data
    if (params.data instanceof Date) {
      _type = 'Date'
    }
    try {
      params.data = JSON.stringify({
        data: params.data,
        _type: _type,
      })
    } catch (error) {
      console.error('parameters data error')
      return
    }
    const data = params || {}
    data.name = 'setStorage'
    data.arg = {
      key: params.key,
      data: params.data,
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * @description 移除本地缓存
   * @param {Object} params
   * @param {String} params.key
   * success: function(res) {},
   * fail: function(res) {},
   * complete: function(res) {},
   */
  removeStorage(params) {
    if (typeof params.key !== 'string') {
      console.error('parameters error')
      return
    }
    const data = params || {}
    data.name = 'removeStorage'
    data.arg = {
      key: params.key,
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * @description 清除所有的缓存
   * success: function(res) {},
   * fail: function(res) {},
   * complete: function(res) {},
   */
  clearStorage(params) {
    const data = params || {}
    data.name = 'clearStorage'
    data.arg = {}
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * @description 获取储存的信息
   * success: function(res) {},
   * fail: function(res) {},
   * complete: function(res) {},
   */
  getStorageInfo(params) {
    const data = params || {}
    data.name = 'getStorageInfo'
    data.arg = {}
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 调用app原生接口
   * @param {*} params {
   *  name: String, // 原生APP方法，命名规范，加【 user_define_ 】前缀
   *  data: Object, // 需要传递给原生APP的数据
   *  success: function(res) {},
   *  fail: function(res) {},
   *  complete: function(res) {},
   * }
   */
  callHandler(params) {
    const data = params || {}
    invokeMethod(params.name, data)
  }

  /**
   * @description 获取当前地理位置信息
   * @param {Object} params
   * @param {String}  params.type 目前不管该参数如何，都会返回gcj02坐标
   * @param {Boolean}  params.altitude 传入 true 会返回高度信息，由于获取高度需要较高精确度，会减慢接口返回速度
   */
  getLocation(params) {
    const data = params || {}
    data.name = 'getLocation'
    data.arg = {
      type: params.type,
      altitude: params.altitude,
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * @description 打开地图位置
   * @param {Object} params
   * @param {Number} params.latitude 目标地纬度
   * @param {Number} params.longitude 目标地经度
   */
  chooseLocation(params) {
    const data = params || {}
    data.name = 'chooseLocation'
    data.arg = {
      latitude: params.latitude,
      longitude: params.longitude,
    }
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
 * @description 请求 api
 * @param {Object}} params
 */
  request(params = {}) {
    if (!params.url || typeof params.url !== 'string') {
      throw new Error('Invalid parameter url')
    }

    const headerType = utils.getDataType(params.header)
    params.header = params.header || {}
    params.header = utils.convertObjectValueToString(params.header)
    if (headerType !== 'Undefined' && headerType !== 'Object') {
      console.warn('request: header must be an object')
      params.header = {}
    }
    params.header = Object.keys(params.header).reduce(function(res, cur) {
      cur.toLowerCase() === 'content-type'
        ? (res[cur.toLowerCase()] = params.header[cur])
        : (res[cur] = params.header[cur])
      return res
    }, {})

    params.method && (params.method = params.method.toUpperCase())
    const headers = params.header || {}
    const requestMethod = typeof params.method === 'string' ? params.method.toUpperCase() : 'GET'
    let data = ''
    params.dataType = params.dataType || 'json'
    headers['content-type'] = headers['content-type'] || 'application/json'
    data = !params.data
      ? ''
      : typeof params.data !== 'string'
        ? headers['content-type'].indexOf(
          'application/x-www-form-urlencoded',
        ) > -1
          ? utils.urlEncodeFormData(params.data, !0)
          : headers['content-type'].indexOf('application/json') > -1
            ? JSON.stringify(params.data)
            : typeof params.data === 'object'
              ? JSON.stringify(params.data)
              : data.toString()
        : params.data

    if (requestMethod === 'GET') {
      params.url = utils.addQueryStringToUrl(params.url, params.data)
    }

    invokeMethod(
      'invokeMiniProgramAPI',
      {
        name: 'request',
        arg: {
          url: params.url,
          data: data,
          header: headers,
          method: requestMethod,
          dataType: params.dataType || 'json',
          responseType: params.responseType,
          taskId: String(Math.random()),
        },
        success: params.success,
        fail: params.fail,
        complete: params.complete,
      },
      {
        beforeSuccess: function(res) {
          if (res.isBase64) {
            if (params.responseType && params.responseType.toLowerCase() === 'arraybuffer') {
              res.data = utils.base64ToArrayBuffer(res.data)
            } else {
              res.data = utils.atob(res.data)
            }
          }
          if (params.dataType === 'json') {
            try {
              res.data = JSON.parse(res.data)
            } catch (e) { }
          }
          res.statusCode = parseInt(res.statusCode)
        },
      },
    )
  }

  getSuperviseInfo(params) {
    const data = params || {}
    data.name = 'getSuperviseInfo'
    data.arg = {}
    invokeMethod('invokeMiniProgramAPI', data)
  }

  /**
   * 注册小程序onShow时回调
   * @param {*} params {
   *
   * }
   */
  onShow(params) {
    const data = params || {}
    subscribe('pageonshow', data)
  }

  onHide(params) {
    const data = params || {}
    subscribe('pageonhide', data)
  }

  onUnload(params) {
    const data = params || {}
    webSubscribe('pageonunload', data)
  }

  /**
   * H5 调用原生方法
   * @param {String} name
   * @param {Object} params
   * @param {Funciton} callback
   */
  callNativeAPI(name, params, callback) {
    invoke(name, params, callback)
  }

  /**
   * 注册 H5 方法供原生调用
   * @param {String} name
   * @param {Function} handler
   */
  registNativeAPIHandler(name, handler) {
    webSubscribe(name, handler)
  }

  /**
   * 监听小程序发送的事件
   * @param {*} handler
   */
  onWebviewEvent(handler) {
    if (typeof handler === 'function') {
      eventHandlers.push(handler)
    }
  }

  /**
   * 移除监听小程序发送的事件
   * @param {*} handler
   */
  offWebviewEvent(handler) {
    if (typeof handler === 'function') {
      eventHandlers = eventHandlers.filter((f) => f !== handler)
    }
  }

  /**
   * 触发 webview 标签绑定的 bindevent 方法
   * @param {*} data
   */
  sendWebviewEvent(data) {
    publish('onWebviewEvent', {
      key: 'sendEvent',
      params: data,
    })
  }
}

const MOPClientInstance = new MOPClient()
window.subscribeHandler = subscribeHandler
window.fc = {
  miniProgram: MOPClientInstance
}
window.ft = window.fc
window.addEventListener('message', function(e) {
  try {
    const { callbackId, params } = JSON.parse(e.data)
    window.FinChatJSBridge.webInvokeCallbackHandler(callbackId, params)
  } catch (e) {

  }
}, false)

export default MOPClientInstance
